"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//
class NotebookInfo {
    /** create an info object from metadata of the notebook
     */
    constructor(notebook) {
        this.notebook = notebook;
        this.KernelList = new Array();
        this.autoResume = false;
        this.sos_comm = null;
        this.BackgroundColor = new Map();
        this.DisplayName = new Map();
        this.KernelName = new Map();
        this.LanguageName = new Map();
        this.KernelOptions = new Map();
        this.CodeMirrorMode = new Map();
        this.pendingCells = new Map();
        let data = [["SoS", "sos", "", ""]];
        if (notebook.model.metadata.has("sos"))
            data = notebook.model.metadata.get("sos")["kernels"];
        // fill the look up tables with language list passed from the kernel
        for (let i = 0; i < data.length; i++) {
            // BackgroundColor is color
            this.BackgroundColor.set(data[i][0], data[i][3]);
            this.BackgroundColor.set(data[i][1], data[i][3]);
            // DisplayName
            this.DisplayName.set(data[i][0], data[i][0]);
            this.DisplayName.set(data[i][1], data[i][0]);
            // Name
            this.KernelName.set(data[i][0], data[i][1]);
            this.KernelName.set(data[i][1], data[i][1]);
            // LanguageName
            this.LanguageName.set(data[i][0], data[i][2]);
            this.LanguageName.set(data[i][1], data[i][2]);
            // if codemirror mode ...
            if (data[i].length >= 5 && data[i][4]) {
                this.CodeMirrorMode.set(data[i][0], data[i][4]);
            }
            this.KernelList.push(data[i][0]);
        }
    }
    updateLanguages(data) {
        for (let i = 0; i < data.length; i++) {
            // BackgroundColor is color
            this.BackgroundColor.set(data[i][0], data[i][3]);
            // by kernel name? For compatibility ...
            if (!(data[i][1] in this.BackgroundColor)) {
                this.BackgroundColor.set(data[i][1], data[i][3]);
            }
            // DisplayName
            this.DisplayName.set(data[i][0], data[i][0]);
            if (!(data[i][1] in this.DisplayName)) {
                this.DisplayName.set(data[i][1], data[i][0]);
            }
            // Name
            this.KernelName.set(data[i][0], data[i][1]);
            if (!(data[i][1] in this.KernelName)) {
                this.KernelName.set(data[i][1], data[i][1]);
            }
            // Language Name
            this.LanguageName.set(data[i][0], data[i][2]);
            if (!(data[i][2] in this.LanguageName)) {
                this.LanguageName.set(data[i][2], data[i][2]);
            }
            // if codemirror mode ...
            if (data[i].length > 4 && data[i][4]) {
                this.CodeMirrorMode.set(data[i][0], data[i][4]);
            }
            // if options ...
            if (data[i].length > 5) {
                this.KernelOptions.set(data[i][0], data[i][5]);
            }
            if (this.KernelList.indexOf(data[i][0]) === -1)
                this.KernelList.push(data[i][0]);
        }
        // add css to window
        let css_text = this.KernelList.map(
        // add language specific css
        (lan) => {
            if (this.BackgroundColor.get(lan)) {
                let css_name = safe_css_name(`sos_lan_${lan}`);
                return `.jp-CodeCell.${css_name} .jp-InputPrompt,
            .jp-CodeCell.${css_name} .jp-OutputPrompt {
              background: ${this.BackgroundColor.get(lan)};
            }
          `;
            }
            else {
                return null;
            }
        })
            .filter(Boolean)
            .join("\n");
        var css = document.createElement("style");
        css.type = "text/css";
        css.innerHTML = css_text;
        document.body.appendChild(css);
    }
    show() {
        console.log(this.KernelList);
    }
}
exports.NotebookInfo = NotebookInfo;
function safe_css_name(name) {
    return name.replace(/[^a-z0-9_]/g, function (s) {
        var c = s.charCodeAt(0);
        if (c == 32)
            return "-";
        if (c >= 65 && c <= 90)
            return "_" + s.toLowerCase();
        return "__" + ("000" + c.toString(16)).slice(-4);
    });
}
exports.safe_css_name = safe_css_name;
class Manager {
    constructor() {
        if (!this._info) {
            this._info = new Map();
        }
    }
    static set_trackers(notebook_tracker, console_tracker) {
        this._notebook_tracker = notebook_tracker;
        this._console_tracker = console_tracker;
    }
    static set_commands(commands) {
        this._commands = commands;
    }
    static get currentNotebook() {
        return this._notebook_tracker.currentWidget;
    }
    static consolesOfNotebook(panel) {
        return this._console_tracker.filter(value => {
            return value.console.session.path === panel.context.path;
        });
    }
    static get currentConsole() {
        return this._console_tracker.currentWidget;
    }
    static get commands() {
        return this._commands;
    }
    static get manager() {
        if (this._instance === null || this._instance === undefined)
            this._instance = new Manager();
        return this._instance;
    }
    // register notebook info to the global registry
    get_info(notebook) {
        if (!this._info.has(notebook)) {
            console.log("Creating a new notebook info");
            this._info.set(notebook, new NotebookInfo(notebook));
        }
        return this._info.get(notebook);
    }
    register_comm(comm, notebook) {
        this.get_info(notebook).sos_comm = comm;
    }
    // this is the same as get_info,
    notebook_of_comm(comm_id) {
        for (let [panel, info] of Array.from(this._info.entries()))
            if (info.sos_comm && info.sos_comm.commId === comm_id)
                return panel;
    }
}
exports.Manager = Manager;
