"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const manager_1 = require("./manager");
const CELL_LANGUAGE_DROPDOWN_CLASS = "jp-CelllanguageDropDown";
function saveKernelInfo() {
    let panel = manager_1.Manager.currentNotebook;
    let info = manager_1.Manager.manager.get_info(panel);
    let used_kernels = new Set();
    let cells = panel.content.model.cells;
    for (var i = cells.length - 1; i >= 0; --i) {
        let cell = cells.get(i);
        if (cell.type === "code" && cell.metadata.get("kernel")) {
            used_kernels.add(cell.metadata.get("kernel"));
        }
    }
    panel.content.model.metadata.get("sos")["kernels"] = Array.from(used_kernels)
        .sort()
        .map(function (x) {
        return [
            info.DisplayName.get(x),
            info.KernelName.get(x),
            info.LanguageName.get(x) || "",
            info.BackgroundColor.get(x) || "",
            info.CodeMirrorMode.get(x) || ""
        ];
    });
}
exports.saveKernelInfo = saveKernelInfo;
function hideLanSelector(cell) {
    let nodes = cell.node.getElementsByClassName(CELL_LANGUAGE_DROPDOWN_CLASS);
    if (nodes.length > 0) {
        nodes[0].style.display = "none";
    }
}
exports.hideLanSelector = hideLanSelector;
function toggleDisplayOutput(cell) {
    if (cell.model.type === "markdown") {
        // switch between hide_output and ""
        if (cell.model.metadata.get("tags") &&
            cell.model.metadata.get("tags").indexOf("hide_output") >= 0) {
            // if report_output on, remove it
            remove_tag(cell, "hide_output");
        }
        else {
            add_tag(cell, "hide_output");
        }
    }
    else if (cell.model.type === "code") {
        // switch between report_output and ""
        if (cell.model.metadata.get("tags") &&
            cell.model.metadata.get("tags").indexOf("report_output") >= 0) {
            // if report_output on, remove it
            remove_tag(cell, "report_output");
        }
        else {
            add_tag(cell, "report_output");
        }
    }
}
exports.toggleDisplayOutput = toggleDisplayOutput;
function remove_tag(cell, tag) {
    let taglist = cell.model.metadata.get('tags');
    let new_list = [];
    for (let i = 0; i < taglist.length; i++) {
        if (taglist[i] != tag) {
            new_list.push(taglist[i]);
        }
    }
    cell.model.metadata.set('tags', new_list);
    let op = cell.node.getElementsByClassName("jp-Cell-outputWrapper");
    for (let i = 0; i < op.length; ++i) {
        op.item(i).classList.remove(tag);
    }
}
function add_tag(cell, tag) {
    let taglist = cell.model.metadata.get('tags');
    if (taglist) {
        taglist.push(tag);
    }
    else {
        taglist = [tag];
    }
    cell.model.metadata.set('tags', taglist);
    let op = cell.node.getElementsByClassName("jp-Cell-outputWrapper");
    for (let i = 0; i < op.length; ++i) {
        op.item(i).classList.add(tag);
    }
}
function addLanSelector(cell, info) {
    if (!cell.model.metadata.has("kernel")) {
        cell.model.metadata.set("kernel", "SoS");
    }
    let kernel = cell.model.metadata.get("kernel");
    let nodes = cell.node.getElementsByClassName(CELL_LANGUAGE_DROPDOWN_CLASS);
    if (nodes.length === 0) {
        // if there is no selector, create one
        let select = document.createElement("select");
        for (let lan of info.KernelList) {
            let option = document.createElement("option");
            option.value = lan;
            option.id = lan;
            option.textContent = lan;
            select.appendChild(option);
        }
        select.className = CELL_LANGUAGE_DROPDOWN_CLASS + " sos-widget";
        let cell_should_be_hidden = false;
        if (cell.inputHidden) {
            // if the cell is collapsed, no jp-InputArea-editor could be found
            cell.inputHidden = false;
            cell_should_be_hidden = true;
            // cell.inputHidden = true;
        }
        let editor = cell.node.getElementsByClassName("jp-InputArea-editor")[0];
        editor.parentElement.insertBefore(select, editor);
        select.value = kernel;
        select.addEventListener("change", function (evt) {
            // set cell level meta data
            let kernel = evt.target.value;
            cell.model.metadata.set("kernel", kernel);
            info.sos_comm.send({ "set-editor-kernel": kernel });
            // change style
            changeStyleOnKernel(cell, kernel, info);
            // set global meta data
            saveKernelInfo();
        });
        if (cell_should_be_hidden) {
            cell.inputHidden = true;
        }
    }
    else {
        // use the existing dropdown box
        let select = nodes.item(0);
        // update existing
        for (let lan of info.KernelList) {
            // ignore if already exists
            if (select.options.namedItem(lan))
                continue;
            let option = document.createElement("option");
            option.value = lan;
            option.id = lan;
            option.textContent = lan;
            select.appendChild(option);
        }
        select.value = kernel ? kernel : "SoS";
    }
}
exports.addLanSelector = addLanSelector;
function changeCellKernel(cell, kernel, info) {
    cell.model.metadata.set("kernel", kernel);
    let nodes = cell.node.getElementsByClassName(CELL_LANGUAGE_DROPDOWN_CLASS);
    // use the existing dropdown box
    let select = nodes.item(0);
    if (select) {
        select.value = kernel;
    }
    changeStyleOnKernel(cell, kernel, info);
}
exports.changeCellKernel = changeCellKernel;
function changeStyleOnKernel(cell, kernel, info) {
    // Note: JupyterLab does not yet support tags
    if (cell.model.metadata.get("tags") &&
        cell.model.metadata.get("tags").indexOf("report_output") >= 0) {
        let op = cell.node.getElementsByClassName("jp-Cell-outputWrapper");
        for (let i = 0; i < op.length; ++i)
            op.item(i).classList.add("report-output");
    }
    else {
        let op = cell.node.getElementsByClassName("jp-Cell-outputWrapper");
        for (let i = 0; i < op.length; ++i)
            op.item(i).classList.remove("report-output");
    }
    for (let className of Array.from(cell.node.classList)) {
        if (className.startsWith("sos_lan_")) {
            cell.node.classList.remove(className);
        }
    }
    cell.node.classList.add(manager_1.safe_css_name(`sos_lan_${kernel}`));
    // cell.user_highlight = {
    //     name: 'sos',
    //     base_mode: info.LanguageName[kernel] || info.KernelName[kernel] || kernel,
    // };
    // //console.log(`Set cell code mirror mode to ${cell.user_highlight.base_mode}`)
    let base_mode = info.CodeMirrorMode.get(kernel) ||
        info.LanguageName.get(kernel) ||
        info.KernelName.get(kernel) ||
        kernel;
    if (!base_mode || base_mode === "sos") {
        cell.inputArea.editorWidget.editor.setOption("mode", "sos");
    }
    else {
        cell.inputArea.editorWidget.editor.setOption("mode", {
            name: "sos",
            base_mode: base_mode
        });
    }
}
exports.changeStyleOnKernel = changeStyleOnKernel;
function updateCellStyles(panel, info) {
    var cells = panel.content.widgets;
    // setting up background color and selection according to notebook metadata
    for (let i = 0; i < cells.length; ++i) {
        addLanSelector(cells[i], info);
        if (cells[i].model.type === "code") {
            changeStyleOnKernel(cells[i], cells[i].model.metadata.get("kernel"), info);
        }
    }
    let panels = manager_1.Manager.consolesOfNotebook(panel);
    for (let i = 0; i < panels.length; ++i) {
        addLanSelector(panels[i].console.promptCell, info);
        changeStyleOnKernel(panels[i].console.promptCell, panels[i].console.promptCell.model.metadata.get("kernel"), info);
    }
    let tasks = document.querySelectorAll('[id^="status_"]');
    let unknownTasks = [];
    for (let i = 0; i < tasks.length; ++i) {
        // status_localhost_5ea9232779ca1959
        if (tasks[i].id.match("^task_status_icon_.+_[0-9a-f]{16,32}$")) {
            tasks[i].className = "fa fa-fw fa-2x fa-refresh fa-spin";
            unknownTasks.push(tasks[i].id);
        }
    }
    return unknownTasks;
}
exports.updateCellStyles = updateCellStyles;
